@@ -290,6 +290,8 @@ typedef struct {
 #define RR_AAAA 28
 /* SRV: "service" records */
 #define RR_SRV 33
+/* SPF: sender policy framework (stops forged email) record */
+#define RR_SPF 99
 /* AXFR: Special query that requests a zone transfer */
 #define RR_AXFR 252
 /* ANY: Special query for all of the records for a given hostname */
