@@ -1,9 +1,26 @@
-/* Placed in the public domain 2001 by Sam Trenholme */
+/* Copyright (c) 2002 Sam Trenholme
+ *
+ * TERMS
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * This software is provided 'as is' with no guarantees of correctness or
+ * fitness for purpose.
+ */
 
 /* Routines for converting varous query data specified in RFC1035 in to
    data to send over the network */
 
 #include "../MaraDns.h"
+#include "functions_dns.h"
 
 /* Determine the length of a domain-name label (if it is a compressed
    label, determine the length of the compressed portion)
@@ -28,7 +45,7 @@ int dlabel_length(js_string *raw, unsigned int offset) {
     if(offset > raw->unit_count)
         return JS_ERROR;
     toread = *(raw->string + offset);
-    while(length < 256 && toread > 0) {
+    while(length < 256 && toread > 0 && toread != '_') {
         if(toread >= 192) /* compressed label */ {
             if(length + 2 + offset <= raw->unit_count)
 	       return length + 2;
@@ -193,7 +210,7 @@ int make_question(q_question *question, js_string *js) {
 
     int offset = js->unit_count;
 
-    char *raw;
+    unsigned char *raw;
      
     counter = toread = 0;
 
@@ -266,7 +283,7 @@ int read_question(js_string *js,q_question *question, int offset) {
 
     unsigned char toread, read;
 
-    char *raw;
+    unsigned char *raw;
      
     counter = toread = 0;
 
@@ -337,9 +354,10 @@ int read_question(js_string *js,q_question *question, int offset) {
 */
 
 int hname_translate(js_string *hostname, int qtype) {
-    int counter, special_root_case = 0;
-    unsigned char toread;
-    char *raw;
+    int counter = 0, special_root_case = 0;
+    unsigned char toread = 0;
+    unsigned char *raw = 0;
+    int dotseen = 0;
 
     /* Sanity checks */
     if(js_has_sanity(hostname) == JS_ERROR)
@@ -386,6 +404,11 @@ int hname_translate(js_string *hostname, int qtype) {
         case -2: /* Special command to make it a pipe */
 	    *raw = '|';
 	    break;
+	case RR_MAGIC_SPACE: /* Special command to make it a space */
+	case RR_MAGIC_EMAIL: /* Same as magic space, but make the first
+                              * seperator an @ instead of a dot */
+	    *raw = ' ';
+	    break;
         default:
 	    *raw = 'U';
         }
@@ -403,6 +426,7 @@ int hname_translate(js_string *hostname, int qtype) {
         
      /* Convert the hostname delimters in to dots */
      counter = 0;
+     dotseen = 0;
      while(counter < 256 && toread > 0) {
          if(toread > 63)
 	     return JS_ERROR; /* To do: compression support */
@@ -412,10 +436,16 @@ int hname_translate(js_string *hostname, int qtype) {
              return JS_ERROR; 
 	 toread = *(raw + counter);
 	 /* Buffer overflow protection */
-	 if(counter <= hostname->unit_count)
-	     *(raw + counter) = '.';
-         else
+	 if(counter <= hostname->unit_count) {
+	     if(qtype != RR_MAGIC_EMAIL || dotseen > 0) {
+	         *(raw + counter) = '.';
+             } else {
+                 *(raw + counter) = '@';
+		 dotseen = 1;
+	     } 
+         } else {
 	     return JS_ERROR;
+             }
          } 
 
      return JS_SUCCESS;
@@ -431,7 +461,7 @@ int email_translate(js_string *hostname) {
     int counter;
     unsigned char toread;
     int first = 1;
-    char *raw;
+    unsigned char *raw;
 
     /* Sanity checks */
     if(js_has_sanity(hostname) == JS_ERROR)
@@ -484,6 +514,13 @@ int email_translate(js_string *hostname) {
 */
 
 int hname_2rfc1035(js_string *hostname) {
+    return hname_2rfc1035_starwhitis(hostname,0);
+    }
+
+/* Starwhitis: 0: Stars *not* allowed at end of host name
+               1: Stars are allowed at host name ends */
+
+int hname_2rfc1035_starwhitis(js_string *hostname, int starwhitis) {
     int counter,seen;
     unsigned char *raw, *towrite;
     int ret = -2;
@@ -539,6 +576,14 @@ int hname_2rfc1035(js_string *hostname) {
 	 return ret;
          }
 
+     /* Also handle the case of the hostname being *just* a star */
+     if(*(raw + 1) == '*' && hostname->unit_count == 2 &&
+		     starwhitis == 1) {
+         *raw = '_';
+         hostname->unit_count = 1;
+	 return ret;
+         }
+
      /* Convert the dots in to hostname delimiters */
      seen = counter = 0;
      while(counter < 256 && counter < hostname->unit_count) {
@@ -559,10 +604,17 @@ int hname_2rfc1035(js_string *hostname) {
          } 
 
      /* That last '0' for the dot after .com */
-     if(counter < 256 && *(raw + counter) == '.')
+     if(counter < 256 && *(raw + counter) == '.') {
          *(raw + counter) = 0;
-     else 
+     /* We need to deal with trailing * characters */
+     } else if(counter < 256 && counter > 1 && *(raw + counter) == '*' &&
+		     hostname->unit_count > 1 && starwhitis == 1) {
+         *(raw + counter - 1) = '_'; 
+	 hostname->unit_count--;
+     }
+     else {
          return JS_ERROR;
+     }
 
      return ret;
      }
@@ -578,7 +630,7 @@ int email_2rfc1035(js_string *hostname) {
      int counter,seen;
      unsigned char *raw, *towrite;
 
-     int firstat;
+     int firstat = 1;
 
      /* Sanity checks */
      if(js_has_sanity(hostname) == JS_ERROR)
@@ -591,26 +643,8 @@ int email_2rfc1035(js_string *hostname) {
      /* Store the length of the first part of the hostname */
      raw = towrite = hostname->string;
 
-     /* If there are no @s in name, but there are dots, make first dot
-        an at.  Too many people are used to BIND's buggy handling of
-	email addresses */
-     seen = 0;
-     firstat = 0;
-     for(counter = 0;counter < hostname->unit_count;counter++) {
-         if(*(hostname->string + counter) == '@' && firstat == 0) {
-	     firstat = counter;
-	     }
-         if(*(hostname->string + counter) == '.' && seen == 0) {
-	     seen = counter;
-	     }
-         }
-     if(seen > 0 && firstat == 0) {
-         *(hostname->string + seen) = '@';
-	 }
-
      /* Convert the at/dots in to hostname delimiters */
      seen = counter = 0;
-     firstat = 1;
      while(counter < 256 && counter < hostname->unit_count) {
          counter++;
          if(counter > hostname->unit_count) 
