@@ -893,6 +893,9 @@ int csv2_numeric_rtype(js_string *text_rtype) {
 	if(js_qissame("txt",text_rtype) == 1) {
 		return RR_TXT;
 	}
+	if(js_qissame("spf",text_rtype) == 1) {
+		return RR_SPF;
+	}
 	if(js_qissame("fqdn4",text_rtype) == 1) {
 		return 65765;
 	}
@@ -1187,6 +1190,7 @@ int csv2_read_rr(csv2_add_state *state, csv2_read *stream, int32 starwhitis) {
 			rddata = csv2_get_srv(stream,zone);
 			break;
 		case RR_TXT:
+		case RR_SPF:
 			rddata = csv2_get_txt(stream);
 			break;
 		case RR_NS:
