@@ -1,6 +1,20 @@
-/* Placed in the public domain 2001 by Sam Trenholme */
-
-/* This is the core DNS server */
+/* Copyright (c) 2002-2006 Sam Trenholme
+ *
+ * TERMS
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * This software is provided 'as is' with no guarantees of correctness or
+ * fitness for purpose.
+ */
 
 /* Langauge-specific labels */
 #include "MaraBigHash_locale.h"
@@ -16,20 +30,25 @@
 #include <string.h>
 #include <time.h>
 #include <sys/types.h>
+#ifndef MINGW32
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#else
+#include <winsock.h>
+#include <wininet.h>
+#endif
 #include "../dns/functions_dns.h"
 #include "../parse/functions_parse.h"
+#include "../parse/Csv2_database.h"
+#include "../parse/Csv2_read.h"
+#include "../parse/Csv2_functions.h"
 #include "functions_server.h"
+#include "timestamp.h"
 
 extern int log_level;
 extern int no_cname_warnings;
 
-extern int mhash_firstkey();
-extern int mhash_nextkey();
-
-extern void human_readable_dns_query();
 
 /* Handler to handle warnings.
    Input: Pointer to js_string object with warning message,
@@ -47,16 +66,16 @@ int warning(js_string *msg, int line) {
     return JS_SUCCESS;
     }
 
-/* Given a js_string object containing a raw UDP dname followed by any
-   data whatsoever (such as, say, a 16-bit big-endian record type),
+/* Given a js_string object containing a raw UDP dname,
    if the data starts off with a "star record" (<01>*), convert the
    star record to a '_'.  There should be no metacharacters in domain
    labels--use EDNS-style labels for metainformation!
-   input: input to label we modify in place
+   input: input to label we modify in place, whether we are recursive
+          (and not permit star labels at the end of a record)
    output: JS_SUCCESS if we were successful, JS_ERROR on error
 */
 
-int starrecord_to_meta(js_string *rr) {
+int starrecord_to_meta(js_string *rr, int recursive) {
 
     int counter;
     if(js_has_sanity(rr) == JS_ERROR)
@@ -78,6 +97,26 @@ int starrecord_to_meta(js_string *rr) {
         rr->unit_count--;
         }
 
+    /* If the dlabel ends with <01>*<00>, return a "no stars
+       at the end" error message (if recursion is enabled), and convert
+       the star label in question */
+    counter = dlabel_length(rr,0);
+    counter -= 3;
+    if(counter < 0)
+        return JS_SUCCESS;
+    if(*(rr->string + counter) == 1 && *(rr->string + counter + 1) == '*') {
+        *(rr->string + counter) = '_';
+        counter++;
+        for(; counter < rr->unit_count; counter++)
+            *(rr->string + counter) = *(rr->string + counter + 2);
+        rr->unit_count -= 2;
+        if(recursive == 1) {
+            show_timestamp();
+            printf(
+"Warning: star labels at the end can not be used when recursion is enabled\n");
+            }
+        }
+
     return JS_SUCCESS;
     }
 
@@ -127,6 +166,25 @@ int change_rtype(js_string *js, int newtype) {
     return JS_SUCCESS;
     }
 
+/* Initialize a rr data structure
+   Input: Pointer to the rr
+   Output: none
+ */
+
+void init_rr(rr *data) {
+     data->expire = data->ttl = data->authoritative = data->rr_type = 0;
+     data->next = data->ip = 0;
+     data->query = data->data = 0;
+     data->ptr = 0;
+     data->seen = 0;
+     data->zap = 0;
+     data->expire = data->ttl = data->authoritative = 0;
+     data->perms = 0;
+     data->list = 0;
+     data->rcode = 0;
+     return;
+     }
+
 /* Add a host (domain) name to the big hash
    Input: Pointer to hash, host name, host ttl, authoritative flag,
           expire (currently always 0)
@@ -134,7 +192,7 @@ int change_rtype(js_string *js, int newtype) {
 */
 
 int mhash_put_rr(mhash *hash, js_string *query, js_string *value, uint32 ttl,
-                 uint32 authoritative, uint32 expire) {
+                 uint32 authoritative, uint32 expire, perm_t perms) {
 
     rr *data = 0;
     js_string *new = 0;
@@ -144,18 +202,13 @@ int mhash_put_rr(mhash *hash, js_string *query, js_string *value, uint32 ttl,
         return JS_ERROR;
 
     /* First, clear out all the fields */
-    data->expire = data->ttl = data->authoritative = data->rr_type = 0;
-    data->next = data->ip = 0;
-    data->query = data->data = 0;
-    data->seen = 0;
-    data->zap = 0;
-    data->ptr = 0;
-    data->rcode = 0;
+    init_rr(data);
 
     /* Store the simple data in the rr data */
     data->expire = 0;
     data->ttl = ttl;
     data->authoritative = authoritative;
+    data->perms = perms;
     /* Get the rr type from the query string */
     rrtype = get_rtype(query);
     if(rrtype == JS_ERROR) {
@@ -187,6 +240,11 @@ int mhash_put_rr(mhash *hash, js_string *query, js_string *value, uint32 ttl,
     /* Store that data in the big hash */
     ret = mhash_put(hash,query,data,MARA_DNSRR);
 
+    /* Add this entry to the corresponding ANY chain */
+    if(any_add_rr(hash,query,data) == JS_ERROR) {
+        return JS_ERROR;
+        }
+
     /* Automatically grow the hash if appropriate */
     mhash_autogrow(hash);
 
@@ -214,10 +272,10 @@ int mhash_put_rr(mhash *hash, js_string *query, js_string *value, uint32 ttl,
 */
 
 int mhash_add_rr(mhash *hash, js_string *query, js_string *value, uint32 ttl,
-                 uint32 authoritative, uint32 expire) {
+                 uint32 authoritative, uint32 expire, perm_t perms) {
 
-    rr *data = 0, *point = 0;
-    js_string *new = 0;
+    rr *data, *point;
+    js_string *new;
     mhash_e spot_data;
     int rrtype;
 
@@ -226,18 +284,13 @@ int mhash_add_rr(mhash *hash, js_string *query, js_string *value, uint32 ttl,
         return JS_ERROR;
 
     /* First, clear out all the fields */
-    data->expire = data->ttl = data->authoritative = data->rr_type = 0;
-    data->next = data->ip = 0;
-    data->query = data->data = 0;
-    data->seen = 0;
-    data->zap = 0;
-    data->ptr = 0;
-    data->rcode = 0;
+    init_rr(data);
 
     /* Store the simple data in the rr data */
     data->expire = 0;
     data->ttl = ttl;
     data->authoritative = authoritative;
+    data->perms = perms;
 
     /* Get the rr type from the query string */
     rrtype = get_rtype(query);
@@ -316,11 +369,6 @@ int mhash_add_ptr(mhash *hash, js_string *query, js_string *value) {
     js_string *new;
     mhash_e spot_data;
 
-    /* "NotFina" means that we do not add this entry to the cache */
-    if(value->unit_count == 7 && strncmp(value->string,"NotFina",8) == 0) {
-        return JS_ERROR;
-	}
-
     /* Create a js_string object to store the raw ptr answer */
     if((new=js_create(value->unit_count + 1,value->unit_size)) == 0) {
         /* Jaakko things that we should add a js_destroy(new); here.
@@ -385,13 +433,7 @@ int mhash_add_ip(mhash *hash, js_string *query, js_string *value) {
         return JS_ERROR;
 
     /* First, clear out all the fields */
-    data->expire = data->ttl = data->authoritative = data->rr_type = 0;
-    data->next = data->ip = 0;
-    data->query = data->data = 0;
-    data->seen = 0;
-    data->zap = 0;
-    data->ptr = 0;
-    data->rcode = 0;
+    init_rr(data);
 
     /* Since we are adding an IP, the rrtype is always RR_A */
     rrtype = RR_A;
@@ -534,7 +576,8 @@ int mhash_add_ns(mhash *hash, js_string *query, js_string *ns) {
     /* Next: Find the NS RR (or RR set) we will point to */
     spot_data = mhash_get(hash,ns);
     if(spot_data.value == 0) { /* If the NS element does not exist */
-        return JS_ERROR;
+        return JS_SUCCESS; /* We may not have a NS record in place yet;
+                              just make it an RR without a NS then */
         }
     /* Exit if the NS data pointed to is *not* a DNS resource record */
     if(spot_data.datatype != MARA_DNSRR) {
@@ -556,15 +599,16 @@ int mhash_add_ns(mhash *hash, js_string *query, js_string *ns) {
 
 /* Populate the main assosciative array (the one where the raw UDP query is
    the key and the value is the answer) with the data from the various
-   csv1 files.
+   csv1 files (this is called from populate_main)
    Input: A pointer to the hash to populate, a pointer to the string to
-          put an error message in
+          put an error message in, whether MaraDNS is being recursive or
+          not
    Ouput: JS_ERROR on error, -2 on parsing error, 0 if we don't
           put anything in the csv hash, JS_SUCCESS on success
    Global vars used: The kvars and dvars
  */
 
-int populate_main(mhash *maintable, js_string *error) {
+int parse_csv1s(mhash *maintable, js_string *error, int recursive) {
     mhash *csvs; /* List of CSV1 files to read */
     js_string *zone;
     js_string *udpzone, *filename, *line, *soaline, *pline;
@@ -611,7 +655,8 @@ int populate_main(mhash *maintable, js_string *error) {
     /* Pass 1: fill up the hash with the AN and NS answers */
 
     /* Get all of the keys in the csv1 hash */
-    csvs = (mhash *)dvar_raw(0);
+    csvs = (mhash *)dvar_raw(dq_keyword2n("csv1")); 
+
     if(csvs == 0) {
         /* A csv1 hash is no longer mandatory, now that MaraDNS has recursive
            capability */
@@ -646,7 +691,7 @@ int populate_main(mhash *maintable, js_string *error) {
                 return -2;
                 }
             /* Add the binary RR_NS to the end of the udpzone string
-               (This make authoritative checks easier) */
+               (This makes authoritative checks easier) */
             if(js_addbyte(udpzone,0) == JS_ERROR)
                 return JS_ERROR;
             if(js_addbyte(udpzone,RR_NS) == JS_ERROR)
@@ -684,6 +729,21 @@ int populate_main(mhash *maintable, js_string *error) {
                 /* Increment the Line Number */
                 linenum++;
 
+                /* Handle blank zone files and zone files with only
+                   SOA and authoritative NS records */
+                if(js_buf_eof(desc) && in_ns && line->unit_count == 0) {
+                    /* Yes, there is the special case of a zone with
+                       no records */
+                    if(js_length(soaline) == 0)
+                        continue;
+                    /* Add the SOA in the special case of the zone
+                       only having SOA and NS records */
+                    if(parse_csv1_line(soaline,query,data,&ttl) != RR_SOA)
+                        return JS_ERROR;
+                    add_rr_to_bighash(maintable,query,data,ttl,udpzone,0);
+                    break;
+                    }
+
                 /* We stop processing this file if we are at the end */
                 if(js_buf_eof(desc) && line->unit_count == 0)
                     break;
@@ -697,7 +757,7 @@ int populate_main(mhash *maintable, js_string *error) {
 
                 /* If the query is a star record, change it to MaraDNS'
                    EDNS-style label for star records */
-                starrecord_to_meta(query);
+                starrecord_to_meta(query,recursive);
 
                 /* Handle error conditions */
                 if(rr_type == JS_ERROR) {
@@ -734,7 +794,6 @@ int populate_main(mhash *maintable, js_string *error) {
                        record knows what NS records to place in the NS
                        section of the answer) */
                     else if(rrnum == 1) {
-                        in_ns = 1;
                         if(js_copy(pline,soaline) == JS_ERROR)
                             return JS_ERROR;
                         }
@@ -755,7 +814,7 @@ int populate_main(mhash *maintable, js_string *error) {
                         /* Add the SOA after the zone's NS records */
                         if(parse_csv1_line(soaline,query,data,&ttl) != RR_SOA)
                             return JS_ERROR;
-                        add_rr_to_bighash(maintable,query,data,ttl,udpzone);
+                        add_rr_to_bighash(maintable,query,data,ttl,udpzone,0);
                         /* Now, add the record we are looking at (the
                            first non-SOA and non-authoritative-NS record) */
                         rr_type = parse_csv1_line(pline,query,data,&ttl);
@@ -776,35 +835,29 @@ int populate_main(mhash *maintable, js_string *error) {
                             warning(error,linenum);
                             continue;
                             }
-                        add_rr_to_bighash(maintable,query,data,ttl,udpzone);
+                        add_rr_to_bighash(maintable,query,data,ttl,udpzone,0);
                         in_ns = 0;
                         }
                     else
-                        add_rr_to_bighash(maintable,query,data,ttl,udpzone);
+                        add_rr_to_bighash(maintable,query,data,ttl,udpzone,0);
                     }
                 }
-            /* Handle blank zone files and zone files with only
-               SOA and authoritative NS records */
-            if(in_ns) {
-                /* Yes, there is the special case of a zone with
-                   no records */
-                if(js_length(soaline) == 0)
-                    continue;
-                /* Add the SOA in the special case of the zone
-                   only having SOA and NS records */
-                if(parse_csv1_line(soaline,query,data,&ttl) != RR_SOA)
-                    return JS_ERROR;
-                add_rr_to_bighash(maintable,query,data,ttl,udpzone);
-                }
-
             js_close(desc);
             }
         } while(mhash_nextkey(csvs,zone) != 0);
+    return JS_SUCCESS;
+}
 
-    /* Pass 2: add the appropriate A records */
-    return add_an(maintable,error);
-
-    }
+/* This program parses all of the zone files and adds the records 
+ * to the MaraDNS database */
+int populate_main(mhash *maintable, js_string *error, int recursive) {
+	/* Pass 1: parse all of the zone files */
+	parse_csv1s(maintable,error,recursive);
+	csv2_parse_main_bighash(maintable,0);
+	
+	/* Pass 2: add the appropriate A records */
+	return add_an(maintable,error);
+}
 
 /* Warn about the existance of a DDIP record
    Input: pointer to query with ddip value
@@ -877,12 +930,12 @@ int make_ddip(mhash *bighash, js_string *query) {
     if((js_ip = js_create(5,1)) == 0)
         return JS_ERROR;
 
-    if(js_str2js(js_ip,ip,4,1) == JS_ERROR) {
+    if(js_str2js(js_ip,(char *)ip,4,1) == JS_ERROR) {
         js_destroy(js_ip);
         return JS_ERROR;
         }
 
-    if (mhash_put_rr(bighash,query,js_ip,8675309,1,0) == JS_ERROR) {
+    if(mhash_put_rr(bighash,query,js_ip,8675309,1,0,0) == JS_ERROR) {
         js_destroy(js_ip);
         return JS_ERROR;
         }
@@ -903,7 +956,7 @@ int make_ddip(mhash *bighash, js_string *query) {
 int add_an(mhash *bighash, js_string *error) {
 
     js_string *query, *a_query;
-    mhash_e qdata, adata;
+    mhash_e qdata = {0,0,0}, adata = {0,0,0};
     rr *record;
     int qtype;
 
@@ -917,10 +970,10 @@ int add_an(mhash *bighash, js_string *error) {
     /* Go through the big hash, looking for MX, NS, and CNAME records */
 
     if(mhash_firstkey(bighash,query) <= 0) {
-	js_destroy(a_query);
-	js_destroy(query);
-	return 0;
-	}
+        js_destroy(a_query);
+        js_destroy(query);
+        return 0;
+        }
 
     do {
         /* Determine if this is a MX, NS, or CNAME query */
@@ -928,9 +981,9 @@ int add_an(mhash *bighash, js_string *error) {
         if(qtype == JS_ERROR)
             continue;
         qdata = mhash_get(bighash,query);
-        /* If it is a MX/NS/CNAME query, proceed to find the assosciated
-           A value (if it exists) */
-        if(qtype == RR_MX || qtype == RR_NS || qtype == RR_CNAME) {
+        /* If it is a MX/NS query, proceed to find the assosciated
+           A (IPV4 address) value (if it exists) */
+        if(qtype == RR_MX || qtype == RR_NS) {
             if(qdata.datatype == MARA_DNSRR) {
                 /* See if we have a corresponding A record for each of
                    the answers */
@@ -956,15 +1009,35 @@ int add_an(mhash *bighash, js_string *error) {
                                 }
                             }
                         }
-		    else if(qtype == RR_CNAME && no_cname_warnings == 0) {
+                    } while ((record = record->next) != 0);
+                }
+	    /* If it is a CNAME query, proceed to find the assosciated
+	     * ANY value */
+            } else if(qtype == RR_CNAME) {
+		/* Look for a corresponding ANY value (which needs to be 
+		 * there) */
+		record = qdata.value;
+                do {
+                    if(answer_ip_query(qtype,record->data,a_query) == JS_ERROR)
+                        continue;
+                    if(record->rr_type != RR_CNAME)
+                        break;
+                    if(change_rtype(a_query,RR_ANY) == JS_ERROR)
+                        continue;
+                    adata = mhash_get(bighash,a_query);
+                    /* If so, point to it */
+                    if(adata.datatype == MARA_DNS_LIST) {
+                        if(record != 0 && adata.value != 0)
+                            record->list = adata.value;
+                        }
+		    else if(no_cname_warnings == 0) {
 			printf("Warning: The CNAME record ");
 			human_readable_dns_query(query,1);
-			printf("does not have an A record.\n"
+			printf(" is a dangling CNAME record.\n"
 "Please read the FAQ entry about FAQ entry about dangling CNAME records.\n");
 		        }
                     } while ((record = record->next) != 0);
-                }
-            }
+	        }
         } while(mhash_nextkey(bighash,query) != 0);
     js_destroy(a_query);
     js_destroy(query);
@@ -1048,12 +1121,18 @@ int answer_ip_query(int qtype, js_string *qu, js_string *ar) {
              data in the database.  If it is authoritative, and authoritative
              data is there, do nothing.
 
+          The "perms" (what set of IPs are allowed to view this record) of
+	  the record in question; if a given perm bit is one, only the IP
+	  corresponding to the perm bit in question is allowed to view this
+	  record.  If it is zero, this is the default value: any IP on
+	  the internet can see the record in question.
+
    Output: JS_ERROR on error, JS_SUCCESS on success
 */
 
 int add_rr_to_bighash(mhash *bighash,
                       js_string *query, js_string *data, uint32 ttl,
-                      js_string *zone) {
+                      js_string *zone, perm_t perms) {
 
     int label_len,has_authority = 0,qtype;
     mhash_e hash_spot;
@@ -1070,6 +1149,14 @@ int add_rr_to_bighash(mhash *bighash,
     if((qtype = get_rtype(query)) == JS_ERROR)
         return JS_ERROR;
 
+    /* We are not allowed to add RR_ANY data to the bighash */
+    if(qtype == RR_ANY)
+	return JS_ERROR;
+
+    /* Nor are we allowed to add "AXFR" nor "IXFR" data to the bighash */
+    if(qtype == 251 /* IXFR */ || qtype == 252 /* AXFR */) 
+        return JS_ERROR;
+
     /* See if the data is authoritative or not */
     /* Change the type so the two strings are the same */
     if(change_rtype(zone,qtype) == JS_ERROR) {
@@ -1116,7 +1203,8 @@ int add_rr_to_bighash(mhash *bighash,
        the first NS record for this zone if this is a non-NS record.  */
     if(hash_spot.value == 0) {
         /* Add the data */
-        if(mhash_put_rr(bighash,query,data,ttl,has_authority,0) == JS_ERROR)
+        if(mhash_put_rr(bighash,query,data,ttl,has_authority,0,perms) 
+			== JS_ERROR)
             return JS_ERROR;
         if(mhash_add_ns(bighash,query,zone) == JS_ERROR)
             return JS_ERROR;
@@ -1137,7 +1225,8 @@ int add_rr_to_bighash(mhash *bighash,
        the beginning in the case of NS records) */
     if((point->authoritative == 1 && has_authority == 1) ||
        (point->authoritative == 0 && has_authority == 0)) {
-        if(mhash_add_rr(bighash,query,data,ttl,has_authority,0) == JS_ERROR)
+        if(mhash_add_rr(bighash,query,data,ttl,has_authority,0,perms) 
+			== JS_ERROR)
             return JS_ERROR;
         return JS_SUCCESS;
         }
@@ -1153,18 +1242,25 @@ int add_rr_to_bighash(mhash *bighash,
         nukeme = (rr *)mhash_undef(bighash,query);
         if(nukeme == 0)
             return JS_ERROR;
+	/* Remove this entry from the corresponding
+	 * ANY chain */
+	if(any_zap_rr(bighash,query,nukeme) == JS_ERROR) {
+	    return JS_ERROR;
+	    }
+	/* Remove the entry once and for all */
         if(js_dealloc(nukeme) == JS_ERROR)
             return JS_ERROR;
 
-
         /* Add the data */
-        if(mhash_put_rr(bighash,query,data,ttl,has_authority,0) == JS_ERROR)
+        if(mhash_put_rr(bighash,query,data,ttl,has_authority,0,perms) 
+			== JS_ERROR)
             return JS_ERROR;
         if(mhash_add_ns(bighash,query,zone) == JS_ERROR)
             return JS_ERROR;
         return JS_SUCCESS;
         }
 
+
     /* If the element exists, and it points to something authoritative, and
        we do not have authority for this record, then do nothing */
     if(point->authoritative == 1 && has_authority == 0)
