@@ -542,6 +542,30 @@ int csv2_compatible_output(js_string *uindata, js_string *query) {
     if(read_hdr(uindata,&header) == JS_ERROR)
         harderror(L_INHEADER_CONV); /* Problem converting inheader */
 
+    /* If not 0, show them the rcode from the remote server */
+    switch(header.rcode) {
+	case 1:
+		printf("# Remote server said: FORMAT ERROR\n");
+		break;
+	case 2:
+		printf("# Remote server said: SERVER FAILURE\n");
+		break;
+	case 3:
+		printf("# Remote server said: NAME ERROR\n");
+		break;
+	case 4:
+		printf("# Remote server said: NOT IMPLEMENTED\n");
+		break;
+	case 5:
+		printf("# Remote server said: REFUSED\n");
+		break;
+	case 0:
+		break;
+	default:
+		printf("# Non-RFC1035 RCODE %d sent from remote server\n",
+                       header.rcode);
+	}
+
     /* Move past the questions */
     place = 12;
     for(counter = 0;counter < header.qdcount;counter++) {
